# -*- coding: utf-8 -*-
u"""
Created on 2015-8-11

@author: cheng.li
"""

import unittest
import math
import numpy as np
import copy
import pickle
import tempfile
import os
from collections import deque
from PyFin.Analysis.SecurityValueHolders import SecurityLatestValueHolder
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingAverage
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingDecay
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingVariance
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingStandardDeviation
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingMax
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingArgMax
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingMin
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingArgMin
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingRank
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingQuantile
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingCount
from PyFin.Analysis.TechnicalAnalysis import SecurityTimeMovingCount
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingCountUnique
from PyFin.Analysis.TechnicalAnalysis import SecurityTimeMovingCountUnique
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingAllTrue
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingAnyTrue
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingSum
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingCountedPositive
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingPositiveAverage
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingPositiveDifferenceAverage
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingNegativeDifferenceAverage
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingRSI
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingLogReturn
from PyFin.Analysis.TechnicalAnalysis import SecurityMovingCorrelation


class TestStatefulTechnicalAnalysis(unittest.TestCase):
    def setUp(self):
        # preparing market data
        np.random.seed(0)
        aaplClose = np.random.randn(1000)
        aaplOpen = np.random.randn(1000)
        aaplVolume = np.random.randint(0, 10, 1000)
        applStamps = stamps = np.cumsum(np.random.randint(1, 10, 1000))
        self.aapl = {'close': aaplClose, 'open': aaplOpen, "volume": aaplVolume, "stamp": applStamps}

        ibmClose = np.random.randn(1000)
        ibmOpen = np.random.randn(1000)
        ibmVolume = np.random.randint(0, 10, 1000)
        ibmStamps = stamps = np.cumsum(np.random.randint(1, 10, 1000))
        self.ibm = {'close': ibmClose, 'open': ibmOpen, "volume": ibmVolume, "stamp": ibmStamps}
        self.dataSet = {'aapl': self.aapl, 'ibm': self.ibm}

    def template_test_deepcopy(self, class_type, **kwargs):
        ma = class_type(**kwargs)
        data = dict(aapl=dict(x=1.),
                    ibm=dict(x=2.))
        data2 = dict(aapl=dict(x=2.),
                     ibm=dict(x=3.))

        ma.push(data)
        ma.push(data2)

        copied = copy.deepcopy(ma)

        for name in ma.value.index():
            self.assertAlmostEqual(ma.value[name], copied.value[name])

        for v in np.random.rand(20):
            data['aapl']['x'] = v
            data['ibm']['x'] = v + 1.
            ma.push(data)
            copied.push(data)

            for name in ma.value.index():
                self.assertAlmostEqual(ma.value[name], copied.value[name])

    def template_test_pickle(self, class_type, **kwargs):
        ma = class_type(**kwargs)

        data = dict(aapl=dict(x=1.),
                    ibm=dict(x=2.))
        data2 = dict(aapl=dict(x=2.),
                     ibm=dict(x=3.))

        ma.push(data)
        ma.push(data2)

        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(ma, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            for name in ma.value.index():
                self.assertAlmostEqual(ma.value[name], pickled.value[name])
        os.unlink(f.name)

        for v in np.random.rand(20):
            data['aapl']['x'] = v
            data['ibm']['x'] = v + 1.
            ma.push(data)
            pickled.push(data)

            for name in ma.value.index():
                self.assertAlmostEqual(ma.value[name], pickled.value[name])

    def testSecurityMovingAverage(self):
        window = 10
        ma1 = SecurityMovingAverage(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = np.mean(self.dataSet[name]['close'][start:(i + 1)])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingDecay(self):
        window = 10
        ma1 = SecurityMovingDecay(window, ['close'])

        def calculate_decay(con, k):
            s = k - len(con) + 1
            c = (k + s) * (k - s + 1) / 2.
            sum_value = 0.
            for w in range(s, k+1):
                i = w - s
                sum_value += w * con[i]
            return sum_value / c

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = calculate_decay(self.dataSet[name]['close'][start:(i + 1)], window)
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingAverageWithErrorValues(self):
        window = 5
        to_average = SecurityLatestValueHolder('open') / SecurityLatestValueHolder('close')
        ma = SecurityMovingAverage(window, to_average)

        container = {'aapl': deque(maxlen=window), 'ibm': deque(maxlen=window)}

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=1.0 if self.aapl['close'][i] >= 1.0 else 0.0,
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma.push(data)

            for name in data:
                res = data[name]['open'] / data[name]['close'] if data[name]['close'] != 0. else np.nan
                if res != np.inf and res != -np.inf and not math.isnan(res):
                    container[name].append(res)

            value = ma.value

            for name in value.index():
                expected = np.mean(container[name])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingAverageDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingAverage, window=10, x=['x'])

    def testSecurityMovingAveragePickle(self):
        self.template_test_pickle(SecurityMovingAverage, window=10, x=['x'])

    def testSecurityMovingVariance(self):
        window = 10
        var = SecurityMovingVariance(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))

            var.push(data)

            if i <= 1:
                continue

            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = var.value
            for name in value.index():
                expected = np.var(self.dataSet[name]['close'][start:(i + 1)]) * (i + 1. - start) / (i - start)
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingVarianceDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingVariance, window=10, x=['x'])

    def testSecurityMovingVariancePickle(self):
        self.template_test_pickle(SecurityMovingVariance, window=10, x=['x'])

    def testSecurityMovingStandardDeviation(self):
        window = 10
        std = SecurityMovingStandardDeviation(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))

            std.push(data)

            if i <= 1:
                continue

            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = std.value
            for name in value.index():
                expected = math.sqrt(
                    np.var(self.dataSet[name]['close'][start:(i + 1)]) * (i + 1. - start) / (i - start))
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected,
                                                                                               calculated))

    def testSecurityMovingStandardDeviationDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingStandardDeviation, window=10, x=['x'])

    def testSecurityMovingStandardDeviationPickle(self):
        self.template_test_pickle(SecurityMovingStandardDeviation, window=10, x=['x'])

    def testSecurityMovingMax(self):
        window = 10
        ma1 = SecurityMovingMax(window, 'close')

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = np.max(self.dataSet[name]['close'][start:(i + 1)])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingArgMax(self):
        window = 10
        ma1 = SecurityMovingArgMax(window, 'close')

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = i - start - np.argmax(self.dataSet[name]['close'][start:(i + 1)])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingMaxDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingMax, window=10, x=['x'])

    def testSecurityMovingMaxPickle(self):
        self.template_test_pickle(SecurityMovingMax, window=10, x=['x'])

    def testSecurityMovingMin(self):
        window = 10
        ma1 = SecurityMovingMin(window, 'close')

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = np.min(self.dataSet[name]['close'][start:(i + 1)])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingArgMin(self):
        window = 10
        ma1 = SecurityMovingArgMin(window, 'close')

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = i - start - np.argmin(self.dataSet[name]['close'][start:(i + 1)])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingMinimumDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingMin, window=10, x='x')

    def testSecurityMovingMinimumPickle(self):
        self.template_test_pickle(SecurityMovingMin, window=10, x='x')

    def testSecurityMovingRank(self):
        window = 10
        mq = SecurityMovingRank(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            mq.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                con = self.dataSet[name]['close'][start:(i + 1)]
                sorted_con = sorted(con)
                expected = sorted_con.index(self.dataSet[name]['close'][i])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingCorrelation(self):
        window = 120
        x = SecurityLatestValueHolder('close')
        y = SecurityLatestValueHolder('open')
        mc = SecurityMovingCorrelation(window, x, y)

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            mc.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mc.value
            for name in value.index():
                xs = self.dataSet[name]['close'][start:(i + 1)]
                ys = self.dataSet[name]['open'][start:(i + 1)]
                expected = np.corrcoef(xs, ys)[0, 1]
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingCorrelationNested(self):
        window = 120
        x = SecurityLatestValueHolder('close')
        y = SecurityLatestValueHolder('open')
        z = x > y
        mc = SecurityMovingCorrelation(window, x, z)

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            mc.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mc.value
            for i, name in enumerate(value.index()):
                if i >= window:
                    xs = self.dataSet[name]['close'][start:(i + 1)]
                    ys = self.dataSet[name]['open'][start:(i + 1)]
                    zs = np.where(xs > ys, 1., 0.)
                    expected = np.corrcoef(xs, zs)[0, 1]
                    calculated = value[name]
                    self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                     'expected:   {1:.12f}\n'
                                                                     'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingQuantile(self):
        window = 10
        mq = SecurityMovingQuantile(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            mq.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                con = self.dataSet[name]['close'][start:(i + 1)]
                sorted_con = sorted(con)
                expected = sorted_con.index(self.dataSet[name]['close'][i]) / (len(sorted_con) - 1.)
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingQuantileDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingQuantile, window=10, x=['x'])

    def testSecurityMovingQuantilePickle(self):
        self.template_test_pickle(SecurityMovingQuantile, window=10, x=['x'])

    def testSecurityMovingCount(self):
        window = 10

        self.aapl['close'] = self.aapl['close'] > 0.
        self.ibm['close'] = self.ibm['close'] > 0.

        mq = SecurityMovingCount(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            mq.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                con = self.dataSet[name]['close'][start:(i + 1)]
                expected = len(con)
                calculated = value[name]
                self.assertEqual(expected, calculated, 'at index {0}\n'
                                                       'expected:   {1}\n'
                                                       'calculated: {2}'.format(i, expected, calculated))

    def testSecurityTimeMovingCount(self):
        window = 60

        self.aapl['close'] = self.aapl['close'] > 0.
        self.ibm['close'] = self.ibm['close'] > 0.

        mq = SecurityTimeMovingCount(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i],
                                  stamp=self.aapl["stamp"][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i],
                                 stamp=self.ibm["stamp"][i]))
            mq.push(data)

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                time_diff = (getattr(self, name)["stamp"][i] - getattr(self, name)["stamp"]) < window
                expected = np.sum(time_diff[:i+1])
                calculated = value[name]
                self.assertEqual(expected, calculated, 'at index {0}\n'
                                                       'expected:   {1}\n'
                                                       'calculated: {2}'.format(i, expected, calculated))

    def testSecurityMovingCountUnique(self):
        window = 10

        mq = SecurityMovingCountUnique(window, ['volume'])

        for i in range(len(self.aapl['volume'])):
            data = dict(aapl=dict(volume=self.aapl['volume'][i]),
                        ibm=dict(volume=self.ibm['volume'][i]))
            mq.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                con = self.dataSet[name]['volume'][start:(i + 1)]
                expected = len(np.unique(con))
                calculated = value[name]
                self.assertEqual(expected, calculated, 'at index {0}\n'
                                                       'expected:   {1}\n'
                                                       'calculated: {2}'.format(i, expected, calculated))

    def testSecurityTimeMovingCountUnique(self):
        window = 60

        self.aapl['close'] = self.aapl['close'] > 0.
        self.ibm['close'] = self.ibm['close'] > 0.

        mq = SecurityTimeMovingCountUnique(window, ['volume'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(volume=self.aapl['volume'][i],
                                  open=self.aapl['open'][i],
                                  stamp=self.aapl["stamp"][i]),
                        ibm=dict(volume=self.ibm['volume'][i],
                                 open=self.ibm['open'][i],
                                 stamp=self.ibm["stamp"][i]))
            mq.push(data)

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                time_diff = (getattr(self, name)["stamp"][i] - getattr(self, name)["stamp"]) < window
                time_diff[i + 1:] = False
                expected = len(np.unique(getattr(self, name)["volume"][time_diff]))
                calculated = value[name]
                self.assertEqual(expected, calculated, 'at index {0}\n'
                                                       'expected:   {1}\n'
                                                       'calculated: {2}'.format(i, expected, calculated))

    def testSecurityMovingAllTrue(self):
        window = 3

        self.aapl['close'] = self.aapl['close'] > 0.
        self.ibm['close'] = self.ibm['close'] > 0.

        mq = SecurityMovingAllTrue(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            mq.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                con = self.dataSet[name]['close'][start:(i + 1)]
                expected = np.all(con)
                calculated = value[name]
                self.assertEqual(expected, calculated, 'at index {0}\n'
                                                       'expected:   {1}\n'
                                                       'calculated: {2}'.format(i, expected, calculated))

    def testSecurityMovingAllTrueDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingAllTrue, window=10, x=SecurityLatestValueHolder('x') > 0.)

    def testSecurityMovingAllTruePickle(self):
        self.template_test_pickle(SecurityMovingAllTrue, window=10, x=SecurityLatestValueHolder('x') > 0.)

    def testSecurityMovingAnyTrue(self):
        window = 3

        self.aapl['close'] = self.aapl['close'] > 0.
        self.ibm['close'] = self.ibm['close'] > 0.

        mq = SecurityMovingAnyTrue(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            mq.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            if i < 1:
                continue

            value = mq.value
            for name in value.index():
                con = self.dataSet[name]['close'][start:(i + 1)]
                expected = np.any(con)
                calculated = value[name]
                self.assertEqual(expected, calculated, 'at index {0}\n'
                                                       'expected:   {1}\n'
                                                       'calculated: {2}'.format(i, expected, calculated))

    def testSecurityMovingAnyTrueDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingAnyTrue, window=10, x=SecurityLatestValueHolder('x') > 0.)

    def testSecurityMovingAnyTruePickle(self):
        self.template_test_pickle(SecurityMovingAnyTrue, window=10, x=SecurityLatestValueHolder('x') > 0.)

    def testSecurityMovingSum(self):
        window = 10
        ma1 = SecurityMovingSum(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = np.sum(self.dataSet[name]['close'][start:(i + 1)])
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingSumDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingSum, window=10, x='x')

    def testSecurityMovingSumPickle(self):
        self.template_test_pickle(SecurityMovingSum, window=10, x='x')

    def testSecurityMovingCountedPositive(self):
        window = 10
        ma1 = SecurityMovingCountedPositive(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                expected = np.sum(self.dataSet[name]['close'][start:(i + 1)] > 0.0)
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingCountedPositiveDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingCountedPositive, window=10, x='x')

    def testSecurityMovingCountedPositivePickle(self):
        self.template_test_pickle(SecurityMovingCountedPositive, window=10, x='x')

    def testSecurityMovingPositiveAverage(self):
        window = 10
        ma1 = SecurityMovingPositiveAverage(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i + 1 - window

            value = ma1.value
            for name in value.index():
                sampled = list(filter(lambda x: x > 0, self.dataSet[name]['close'][start:(i + 1)]))
                if len(sampled) > 0:
                    expected = np.mean(sampled)
                else:
                    expected = 0.0
                calculated = value[name]
                self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                 'expected:   {1:.12f}\n'
                                                                 'calculated: {2:.12f}'.format(i, expected, calculated))

    def testSecurityMovingPositiveAverageDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingPositiveAverage, window=10, x='x')

    def testSecurityMovingPositiveAveragePickle(self):
        self.template_test_pickle(SecurityMovingPositiveAverage, window=10, x='x')

    def testSecurityMovingRSI(self):
        window = 10
        rsi = SecurityMovingRSI(window, ['close'])
        pos_avg = SecurityMovingPositiveDifferenceAverage(window, ['close'])
        neg_avg = SecurityMovingNegativeDifferenceAverage(window, ['close'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.aapl['close'][i],
                                  open=self.aapl['open'][i]),
                        ibm=dict(close=self.ibm['close'][i],
                                 open=self.ibm['open'][i]))
            rsi.push(data)
            pos_avg.push(data)
            neg_avg.push(data)

            value = rsi.value
            if i > 0:
                for name in value.index():
                    expected = pos_avg.value[name] / (pos_avg.value[name] - neg_avg.value[name]) * 100
                    calculated = value[name]
                    self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                     'expected:   {1:.12f}\n'
                                                                     'calculated: {2:.12f}'.format(i,
                                                                                                   expected,
                                                                                                   calculated))

    def testSecurityMovingRSIDeepcopy(self):
        self.template_test_deepcopy(SecurityMovingRSI, window=10, x='x')

    def testSecurityMovingRSIPickle(self):
        self.template_test_pickle(SecurityMovingRSI, window=10, x='x')

    def testSecurityMovingLogReturn(self):
        window = 10
        ma1 = SecurityMovingLogReturn(window, ['close'])
        self.newDataSet = copy.deepcopy(self.dataSet)
        self.newDataSet['aapl']['close'] = np.exp(self.newDataSet['aapl']['close'])
        self.newDataSet['aapl']['open'] = np.exp(self.newDataSet['aapl']['open'])
        self.newDataSet['ibm']['close'] = np.exp(self.newDataSet['ibm']['close'])
        self.newDataSet['ibm']['open'] = np.exp(self.newDataSet['ibm']['open'])

        for i in range(len(self.aapl['close'])):
            data = dict(aapl=dict(close=self.newDataSet['aapl']['close'][i],
                                  open=self.newDataSet['aapl']['open'][i]),
                        ibm=dict(close=self.newDataSet['ibm']['close'][i],
                                 open=self.newDataSet['ibm']['open'][i]))
            ma1.push(data)
            if i < window:
                start = 0
            else:
                start = i - window

            value = ma1.value
            for name in value.index():
                sampled = self.newDataSet[name]['close'][start:(i + 1)]
                if i >= 10:
                    expected = math.log(sampled[-1] / sampled[0])
                    calculated = value[name]
                    self.assertAlmostEqual(expected, calculated, 12, 'at index {0}\n'
                                                                     'expected:   {1:.12f}\n'
                                                                     'calculated: {2:.12f}'.format(i, expected,
                                                                                                   calculated))

    def testSecurityMovingLogReturnDeepcopy(self):
        ma = SecurityMovingLogReturn(10, ['x'])

        data = dict(aapl=dict(x=1.),
                    ibm=dict(x=2.))
        data2 = dict(aapl=dict(x=2.),
                     ibm=dict(x=3.))

        ma.push(data)
        ma.push(data2)

        copied = copy.deepcopy(ma)

        for i, v in enumerate(np.random.rand(20)):
            data['aapl']['x'] = v
            data['ibm']['x'] = v + 1.
            ma.push(data)
            copied.push(data)
            if i >= 10:
                for name in ma.value.index():
                    self.assertAlmostEqual(ma.value[name], copied.value[name])

    def testSecurityMovingLogReturnPickle(self):
        ma = SecurityMovingLogReturn(10, ['x'])

        data = dict(aapl=dict(x=1.),
                    ibm=dict(x=2.))
        data2 = dict(aapl=dict(x=2.),
                     ibm=dict(x=3.))

        ma.push(data)
        ma.push(data2)

        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(ma, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
        os.unlink(f.name)

        for i, v in enumerate(np.random.rand(20)):
            data['aapl']['x'] = v
            data['ibm']['x'] = v + 1.
            ma.push(data)
            pickled.push(data)

            if i >= 10:
                for name in ma.value.index():
                    self.assertAlmostEqual(ma.value[name], pickled.value[name])

    def testValueHolderCompounding(self):
        window = 10
        ma1 = SecurityMovingAverage(window, 'close')
        compounded1 = SecurityMovingMax(2, ma1)
        compounded2 = SecurityMovingAverage(2, ma1)

        self.assertEqual(compounded1.window, window + 2)

        container = [np.nan, np.nan]
        for i in range(len(self.aapl['close'])):
            data = {'aapl': {'close': self.aapl['close'][i], 'open': self.aapl['open'][i]}}
            ma1.push(data)
            compounded1.push(data)
            compounded2.push(data)

            container[i % 2] = ma1.value['aapl']

            if i >= 1:
                self.assertAlmostEqual(max(container), compounded1.value['aapl'], 12)
                self.assertAlmostEqual(np.mean((container)), compounded2.value['aapl'], 12)



# -*- coding: utf-8 -*-
u"""
Created on 2015-8-8

@author: cheng.li
"""

from PyFin.tests.Math.Accumulators.testAccumulatorImpl import TestAccumulatorImpl
from PyFin.tests.Math.Accumulators.testAccumulatorsArithmetic import TestAccumulatorsArithmetic
from PyFin.tests.Math.Accumulators.testStatelessAccumulators import TestStatelessAccumulators
from PyFin.tests.Math.Accumulators.testStatefulAccumulators import TestStatefulAccumulators
from PyFin.tests.Math.Accumulators.testPerformancers import TestPerformancers

__all__ = ['TestAccumulatorImpl',
           'TestAccumulatorsArithmetic',
           'TestStatelessAccumulators',
           'TestStatefulAccumulators',
           'TestPerformancers']



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-7

@author: cheng.li
"""

from PyFin.Env.Settings import Settings


class CashFlow(object):

    def __init__(self):
        pass

    def date(self):
        pass

    def amount(self):
        pass

    def exCouponDate(self):
        return None

    def __le__(self, right):
        return self.date() <= right.date()

    def __lt__(self, right):
        return self.date() < right.date()

    def __ge__(self, right):
        return self.date() >= right.date()

    def __gt__(self, right):
        return self.date() > right.date()

    def __eq__(self, right):
        return self.date() == right.date()

    def tradingExCoupon(self, ref_date=None):
        ecd = self.exCouponDate()

        if not ecd:
            return False
        else:
            ref_date = ref_date if ref_date else Settings.evaluationDate
            return ecd <= ref_date

    def hasOccurred(self, ref_date=None, include_ref_date=False):
        if ref_date:
            cf = self.date()
            if ref_date < cf:
                return False
            elif cf < ref_date:
                return True

        if (not ref_date) or ref_date == Settings.evaluationDate:
            ref_date = Settings.evaluationDate
            include_today = Settings.includeTodaysCashFlows

            if include_today:
                include_ref_date = include_today

        if include_ref_date:
            return self.date() < ref_date
        else:
            return self.date() <= ref_date


class SimpleCashFlow(CashFlow):

    def __init__(self, amount, date):
        self._amount = amount
        self._date = date

    def date(self):
        return self._date

    def amount(self):
        return self._amount


class Redemption(SimpleCashFlow):

    def __init__(self, amount, date):
        super(Redemption, self).__init__(amount, date)


class Coupon(CashFlow):

    def __init__(self,
                 paymentDate,
                 norminal,
                 accrualStartDate,
                 accrualEndDate,
                 refPeriodStart=None,
                 refPeriodEnd=None,
                 exCouponDate=None):
        self.paymentDate_ = paymentDate
        self.norminal_ = norminal
        self.accrualStartDate_ = accrualStartDate
        self.accrualEndDate_ = accrualEndDate
        self.refPeriodStart_ = refPeriodStart
        self.refPeriodEnd_ = refPeriodEnd
        self.exCouponDate_ = exCouponDate
        self.accrualPeriod_ = None

        if not self.refPeriodStart_:
            self.refPeriodStart_ = self.accrualStartDate_

        if not self.refPeriodEnd_:
            self.refPeriodEnd_ = self.accrualEndDate_

    def dayCounter(self):
        pass

    def accrualPeriod(self):
        if not self.accrualPeriod_:
            self.accrualPeriod_ = self.dayCounter().yearFraction(self.accrualStartDate_,
                                                                 self.accrualEndDate_,
                                                                 self.refPeriodStart_,
                                                                 self.refPeriodEnd_)
        return self.accrualPeriod_

    def accrualDays(self):
        return self.dayCounter().dayCount(self.accrualStartDate_,
                                          self.accrualEndDate_)

    def accruedPeriodByDate(self, d):
        if d <= self.accrualStartDate_ or d > self.paymentDate_:
            return 0.
        else:
            return self.dayCounter().yearFraction(self.accrualStartDate_,
                                                  min(d, self.accrualEndDate_),
                                                  self.refPeriodStart_,
                                                  self.refPeriodEnd_)

    def accrualDaysByDate(self, d):
        if d <= self.accrualStartDate_ or d > self.paymentDate_:
            return 0
        else:
            return self.dayCounter().dayCount(self.accrualStartDate_,
                                              min(d, self.accrualEndDate_))



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-23

@author: cheng.li
"""

from PyFin.tests.PricingEngines.testBlackFormula import TestBlackFormula
from PyFin.tests.PricingEngines.testSabrFormula import TestSabrFormula
from PyFin.tests.PricingEngines.testSVIInterpolation import TestSVIInterpolation

__all__ = ['TestBlackFormula',
           'TestSabrFormula',
           'TestSVIInterpolation']



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-23

@author: cheng.li
"""

from PyFin.Math import Accumulators
from PyFin.Math import Distributions
from PyFin.Math import MathConstants

__all__ = ['Accumulators',
           'Distributions',
           'MathConstants']



# -*- coding: utf-8 -*-
u"""
Created on 2017-2-10

@author: cheng.li
"""

cpdef int require(condition, exception, str msg=*) except -1

cpdef int ensureRaise(exception, str msg=*) except -1

cpdef int warning(condition, warn_type, str msg=*)

cpdef bint isClose(double a, double b=*, double rel_tol=*, double abs_tol=*) nogil


# -*- coding: utf-8 -*-
u"""
Created on 2017-1-17

@author: cheng.li
"""

import math
import numpy as np


def gediVolatility(strike, forward, expiry, atmVol, skew, kurt):

    moneyness = math.log(strike / forward) / math.sqrt(expiry)
    s = math.atan(moneyness)

    return atmVol + s * skew + kurt * (s ** 2)


def gediVolatilities(strikes, forward, expiry, atmVol, skew, kurt):
    return np.array([gediVolatility(strike, forward, expiry, atmVol, skew, kurt) for strike in strikes])


# -*- coding: utf-8 -*-
u"""
Created on 2015-7-9

@author: cheng.li
"""

from PyFin.Enums.BizDayConventions import BizDayConventions
from PyFin.Enums.DateGeneration import DateGeneration
from PyFin.Enums.Factors import Factors
from PyFin.Enums.Months import Months
from PyFin.Enums.NormalizingType import NormalizingType
from PyFin.Enums.OptionType import OptionType
from PyFin.Enums.TimeUnits import TimeUnits
from PyFin.Enums.Weekdays import Weekdays

__all__ = ['BizDayConventions',
           'DateGeneration',
           'Factors',
           'Months',
           'NormalizingType',
           'OptionType',
           'TimeUnits',
           'Weekdays']



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-8

@author: cheng.li
"""

import unittest
from PyFin.DateUtilities.Date import Date
from PyFin.CashFlows.CashFlow import SimpleCashFlow


class TestCashFlow(unittest.TestCase):

    def testSimpleCashFlow(self):
        cash_flow = SimpleCashFlow(100., Date(2017, 3, 28))

        self.assertAlmostEqual(cash_flow.amount(), 100.)
        self.assertEqual(cash_flow.date(), Date(2017, 3, 28))

    def testSimpleCashFlowComparing(self):
        cf1 = SimpleCashFlow(100., Date(2017, 3, 28))
        cf2 = SimpleCashFlow(100., Date(2017, 4, 1))

        self.assertTrue(cf1 < cf2)
        self.assertFalse(cf1 > cf2)

        cf2 = SimpleCashFlow(100., Date(2017, 3, 1))
        self.assertTrue(cf1 > cf2)
        self.assertFalse(cf1 < cf2)

        cf2 = SimpleCashFlow(100., Date(2017, 3, 28))
        self.assertTrue(cf1 == cf2)

    def testSimpleCashFlowHasOccurred(self):

        cf_date = Date.todaysDate() - 1

        cf1 = SimpleCashFlow(100., cf_date)
        self.assertTrue(cf1.hasOccurred())

        ref_date = cf_date - 1
        self.assertFalse(cf1.hasOccurred(ref_date))

        ref_date = cf_date
        self.assertTrue(cf1.hasOccurred(ref_date))
        self.assertFalse(cf1.hasOccurred(ref_date, include_ref_date=True))


if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
u"""
Created on 2017-1-6

@author: cheng.li
"""

import unittest
import numpy as np
import pandas as pd
from PyFin.Enums import Factors
from PyFin.Analysis.SecurityValueHolders import SecurityLatestValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSRankedSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSTopNSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSBottomNSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSTopNPercentileSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSBottomNPercentileSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSAverageSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSAverageAdjustedSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSZScoreSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSFillNASecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSPercentileSecurityValueHolder
from PyFin.Analysis.CrossSectionValueHolders import CSResidueSecurityValueHolder


class TestCrossSectionValueHolder(unittest.TestCase):

    def setUp(self):
        np.random.seed(0)
        sample1 = np.random.randn(1000, 6)
        sample2 = np.random.randn(1000, 6)

        self.datas = {'aapl': {'close': sample1[:, 0], 'open': sample1[:, 1]},
                      'ibm': {'close': sample2[:, 0], 'open': sample2[:, 1]},
                      'goog': {'close': sample1[:, 2], 'open': sample1[:, 3]},
                      'baba': {'close': sample2[:, 2], 'open': sample2[:, 3]},
                      'tela': {'close': sample1[:, 4], 'open': sample1[:, 5]},
                      'nflx': {'close': sample2[:, 4], 'open': sample2[:, 5]}
                      }

    def testCSRankedSecurityValueHolderWithSymbolName(self):
        benchmark = SecurityLatestValueHolder(x='close')
        rankHolder = CSRankedSecurityValueHolder('close')

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
            benchmark.push(data)
            rankHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal(benchmarkValues.rank().values, rankHolder.value.values)

    def testCSRankedSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        rankHolder = CSRankedSecurityValueHolder(benchmark)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
            benchmark.push(data)
            rankHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal(benchmarkValues.rank().values, rankHolder.value.values)

    def testCSTopNSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        n = 2
        topnHolder = CSTopNSecurityValueHolder(benchmark, n)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['goog'][Factors.OPEN][i]},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i]}}
            benchmark.push(data)
            topnHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal((-benchmarkValues).rank().values <= n, topnHolder.value.values)

    def testCSBottomNSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        n = 2
        topnHolder = CSBottomNSecurityValueHolder(benchmark, n)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['goog'][Factors.OPEN][i]},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i]}}
            benchmark.push(data)
            topnHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal(benchmarkValues.rank().values <= n, topnHolder.value.values)

    def testCSRankedSecurityValueHolderWithGroups(self):
        benchmark = SecurityLatestValueHolder(x='close')
        groups = SecurityLatestValueHolder(x='ind')
        rankHolder = CSRankedSecurityValueHolder(benchmark, groups)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
                             'ind': 1.},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
                            'ind': 1.},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
                             'ind': 2.},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
                             'ind': 2.}}
            benchmark.push(data)
            rankHolder.push(data)
            benchmarkValues = benchmark.value
            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups).rank().values
            np.testing.assert_array_almost_equal(expected_rank, rankHolder.value.values)

    def testCSAverageSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        meanHolder = CSAverageSecurityValueHolder(benchmark)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
            benchmark.push(data)
            meanHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal(benchmarkValues.values.mean(), meanHolder.value.values)

    def testCSAverageSecurityValueHolderWithGroup(self):
        benchmark = SecurityLatestValueHolder(x='close')
        groups = SecurityLatestValueHolder(x='ind')
        meanHolder = CSAverageSecurityValueHolder(benchmark, groups)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
                             'ind': 1.},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
                            'ind': 1.},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
                             'ind': 2.},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
                             'ind': 2.}}
            benchmark.push(data)
            meanHolder.push(data)
            benchmarkValues = benchmark.value
            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
            expected_mean = pd.Series(benchmarkValues.to_dict()).groupby(groups).mean()
            calculated_mean = meanHolder.value

            for name in calculated_mean.index():
                if name in ['aapl', 'ibm']:
                    self.assertAlmostEqual(calculated_mean[name], expected_mean[1])
                else:
                    self.assertAlmostEqual(calculated_mean[name], expected_mean[2])

    def testCSPercentileSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        perHolder = CSPercentileSecurityValueHolder(benchmark)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i]},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['baba'][Factors.OPEN][i]}
                    }
            benchmark.push(data)
            perHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal(benchmarkValues.rank().values / len(data), perHolder.value.values)

    def testCSTopNPercentileSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        n = 0.3
        perHolder = CSTopNPercentileSecurityValueHolder(benchmark, n)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i]},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['baba'][Factors.OPEN][i]}
                    }
            benchmark.push(data)
            perHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal(((-benchmarkValues).rank().values / len(data)) <= n,
                                                 perHolder.value.values)

    def testCSBottomNPercentileSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        n = 0.3
        perHolder = CSBottomNPercentileSecurityValueHolder(benchmark, n)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i]},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['baba'][Factors.OPEN][i]}
                    }
            benchmark.push(data)
            perHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal((benchmarkValues.rank().values / len(data)) <= n,
                                                 perHolder.value.values)

    def testCSPercentileSecurityValueHolderWithGroups(self):
        benchmark = SecurityLatestValueHolder(x='close')
        groups = SecurityLatestValueHolder(x='ind')
        perHolder = CSPercentileSecurityValueHolder(benchmark, groups)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
                             'ind': 1.},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
                            'ind': 1.},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
                             'ind': 2.},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
                             'ind': 2.}}
            benchmark.push(data)
            perHolder.push(data)
            benchmarkValues = benchmark.value
            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups) \
                .transform(lambda x: x.rank().values / len(x))
            np.testing.assert_array_almost_equal(expected_rank, perHolder.value.values)

    def testCSAverageAdjustedSecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        meanAdjustedHolder = CSAverageAdjustedSecurityValueHolder(benchmark)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]},
                    }
            benchmark.push(data)
            meanAdjustedHolder.push(data)
            benchmarkValues = benchmark.value
            np.testing.assert_array_almost_equal((benchmarkValues - benchmarkValues.mean()).values, meanAdjustedHolder.value.values)

    def testCSAverageAdjustedSecurityValueHolderWithGroups(self):
        benchmark = SecurityLatestValueHolder(x='close')
        groups = SecurityLatestValueHolder(x='ind')
        meanAdjustedHolder = CSAverageAdjustedSecurityValueHolder(benchmark, groups)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
                             'ind': 1.},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
                            'ind': 1.},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
                             'ind': 2.},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
                             'ind': 2.}}
            benchmark.push(data)
            meanAdjustedHolder.push(data)
            benchmarkValues = benchmark.value
            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups) \
                .transform(lambda x: x - x.mean())
            np.testing.assert_array_almost_equal(expected_rank, meanAdjustedHolder.value.values)

    def testCSZscoreSecurityValueHolder(self):
        keys = list(range(1, 11))
        values = list(range(10, 0, -1))

        data = {}

        for i, k in enumerate(keys):
            data[k] = {}
            data[k]['close'] = values[i]

        quantile_value = CSZScoreSecurityValueHolder('close')
        quantile_value.push(data)

        calculated = quantile_value.value

        data = np.linspace(10., 1., 10)
        expected = (data - data.mean()) / data.std()

        np.testing.assert_array_almost_equal(expected, calculated.values)

    def testCSFillNASecurityValueHolder(self):
        benchmark = SecurityLatestValueHolder(x='close')
        groups = SecurityLatestValueHolder(x='ind')
        meanAdjustedHolder = CSFillNASecurityValueHolder(benchmark, groups)

        def cal_func(x):
            x[np.isnan(x)] = np.nanmean(x)
            return x

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
                             'ind': 1.},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
                            'ind': 1.},
                    'tela': {Factors.CLOSE: np.nan,
                            Factors.OPEN: self.datas['tela'][Factors.OPEN][i],
                            'ind': 1.},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
                             'ind': 2.},
                    'baba': {Factors.CLOSE: np.nan,
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
                             'ind': 2.},
                    'nflx': {Factors.CLOSE: self.datas['nflx'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['nflx'][Factors.OPEN][i],
                             'ind': 2.}
                    }
            benchmark.push(data)
            meanAdjustedHolder.push(data)
            benchmarkValues = benchmark.value
            groups = {'aapl': 1., 'ibm': 1., 'tela': 1., 'goog': 2., 'baba': 2., 'nflx': 2.}
            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups) \
                .transform(cal_func)
            np.testing.assert_array_almost_equal(expected_rank, meanAdjustedHolder.value.values)

    def testCSZscoreSecurityValueHolderWithGroups(self):
        benchmark = SecurityLatestValueHolder(x='close')
        groups = SecurityLatestValueHolder(x='ind')
        meanAdjustedHolder = CSZScoreSecurityValueHolder(benchmark, groups)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i],
                             'ind': 1.},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i],
                            'ind': 1.},
                    'goog': {Factors.CLOSE: self.datas['goog'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['goog'][Factors.OPEN][i],
                             'ind': 2.},
                    'baba': {Factors.CLOSE: self.datas['baba'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['baba'][Factors.OPEN][i],
                             'ind': 2.}}
            benchmark.push(data)
            meanAdjustedHolder.push(data)
            benchmarkValues = benchmark.value
            groups = {'aapl': 1., 'ibm': 1., 'goog': 2., 'baba': 2.}
            expected_rank = pd.Series(benchmarkValues.to_dict()).groupby(groups) \
                .transform(lambda x: (x - x.mean()) / x.std(ddof=0))
            np.testing.assert_array_almost_equal(expected_rank, meanAdjustedHolder.value.values)

    def testCSZResidueSecurityValueHolder(self):
        y = SecurityLatestValueHolder(x='close')
        x = SecurityLatestValueHolder(x='open')

        res = CSResidueSecurityValueHolder(y, x)

        for i in range(len(self.datas['aapl']['close'])):
            data = {'aapl': {Factors.CLOSE: self.datas['aapl'][Factors.CLOSE][i],
                             Factors.OPEN: self.datas['aapl'][Factors.OPEN][i]},
                    'ibm': {Factors.CLOSE: self.datas['ibm'][Factors.CLOSE][i],
                            Factors.OPEN: self.datas['ibm'][Factors.OPEN][i]}}
            y.push(data)
            x.push(data)
            res.push(data)

            calculated = res.value.values
            y_values = y.value.values
            x_values = x.value.values
            x_values = np.concatenate([np.ones(shape=(len(x_values), 1)), x_values.reshape(-1, 1)], axis=1)

            beta = np.dot(np.linalg.inv(np.dot(x_values.T, x_values)), np.dot(x_values.T, y_values.reshape(-1, 1)))
            expected = y_values - np.dot(x_values, beta).flatten()
            np.testing.assert_array_almost_equal(calculated, expected)



# -*- coding: utf-8 -*-
u"""
Created on 2017-1-8

@author: cheng.li
"""

cimport cython
from libc.math cimport sqrt
from libc.math cimport log
import numpy as np
cimport numpy as np

@cython.cdivision(True)
cpdef double sviVolatility(double strike,
                               double forward,
                               double expiry,
                               double a,
                               double b,
                               double sigma,
                               double rho,
                               double m) nogil:
    k = log(strike / forward)
    totalVairance = a + b * (rho * (k - m) + sqrt((k - m) * (k - m) + sigma * sigma))
    return sqrt(totalVairance / expiry)

@cython.boundscheck(False)
@cython.wraparound(False)
cpdef np.ndarray[double, ndim=1]  sviVolatilities(double[:] strikes,
                                     double forward,
                                     double expiry,
                                     double a,
                                     double b,
                                     double sigma,
                                     double rho,
                                     double m):
    cdef int i
    cdef size_t length = len(strikes)
    cdef np.ndarray[double, ndim=1] res = np.empty(length, np.float64)

    for i in range(length):
        res[i] = sviVolatility(strikes[i], forward, expiry, a, b, sigma, rho, m)
    return res


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef np.ndarray[double, ndim=1] _sviCalibrationIteration(double[:] parameters,
                             list parametetsNames,
                             double[:] strikes,
                             double[:] targetVols,
                             double forward,
                             double expiryTime,
                             dict argsDict):

    cdef int i
    cdef str name

    for i, name in enumerate(parametetsNames):
        argsDict[name] = parameters[i]
    return targetVols - sviVolatilities(strikes,
                                        forward,
                                        expiryTime,
                                        **argsDict)


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef _parametersCheck(double initialA,
                     double initialB,
                     double initialSigma,
                     double initialRho,
                     double initialM,
                     bint isFixedA,
                     bint isFixedB,
                     bint isFixedSigma,
                     bint isFixedRho,
                     bint isFixedM):
    cdef list x0 = []
    cdef list freeParameters = []
    cdef dict fixedParameters = {}
    cdef tuple bounds = ([], [])

    if isFixedA:
        fixedParameters['a'] = initialA
    else:
        freeParameters.append('a')
        x0.append(initialA)
        bounds[0].append(-np.inf)
        bounds[1].append(np.inf)

    if isFixedB:
        fixedParameters['b'] = initialB
    else:
        freeParameters.append('b')
        x0.append(initialB)
        bounds[0].append(0.)
        bounds[1].append(np.inf)

    if isFixedSigma:
        fixedParameters['sigma'] = initialSigma
    else:
        freeParameters.append('sigma')
        x0.append(initialSigma)
        bounds[0].append(0.)
        bounds[1].append(np.inf)

    if isFixedRho:
        fixedParameters['rho'] = initialRho
    else:
        freeParameters.append('rho')
        x0.append(initialRho)
        bounds[0].append(-1.)
        bounds[1].append(1.0)

    if isFixedM:
        fixedParameters['m'] = initialM
    else:
        freeParameters.append('m')
        x0.append(initialRho)
        bounds[0].append(-np.inf)
        bounds[1].append(np.inf)

    return np.array(x0), freeParameters, fixedParameters, bounds




# -*- coding: utf-8 -*-
u"""
Created on 2015-8-11

@author: cheng.li
"""

from PyFin.tests.Analysis.DataProviders import TestDataProviders
from PyFin.tests.Analysis.testSecurityValueHolders import TestSecurityValueHolders
from PyFin.tests.Analysis.testCrossSectionValueHolders import TestCrossSectionValueHolder
from PyFin.tests.Analysis.testSecurityValues import TestSecurityValues
from PyFin.tests.Analysis.testTransformer import TestTransformer
from PyFin.tests.Analysis import TechnicalAnalysis

__all__ = ['TestDataProviders',
           'TestSecurityValueHolders',
           'TestCrossSectionValueHolder',
           'TestSecurityValues',
           'TestTransformer',
           'TestTechnicalAnalysis']



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-8

@author: cheng.li
"""

from PyFin.tests.CashFlows.testCashFlow import TestCashFlow
from PyFin.tests.CashFlows.testInterestRate import TestInterestRate


__all__ = ['TestCashFlow',
           'TestInterestRate']


# -*- coding: utf-8 -*-
u"""
Created on 2017-1-30

@author: cheng.li
"""

cdef public enum Weekdays:
    Sunday = 1
    Monday = 2
    Tuesday = 3
    Wednesday = 4
    Thursday = 5
    Friday = 6
    Saturday = 7


# -*- coding: utf-8 -*-
u"""
Created on 2015-8-19

@author: cheng.li
"""


import unittest
from PyFin.Analysis.DataProviders import DataProvider


class TestDataProviders(unittest.TestCase):

    def testDataProvider(self):
        dataSource = DataProvider()

        dataSource.set('aapl', 'close', 2.0)
        dataSource.set('ibm', 'open', 3.0)
        dataSource.set('goog', 'pe', 4.0)
        dataSource.set('aapl', 'rsi', 5.0)

        calculated = dataSource.data
        expected = {
            'aapl': {'close': 2.0, 'rsi': 5.0},
            'ibm': {'open': 3.0},
            'goog': {'pe': 4.0}
        }

        for name in expected:
            for field in expected[name]:
                eres = expected[name][field]
                cres = calculated[name][field]
                self.assertEqual(eres, cres, "for {0}'s field {1}\n"
                                             "expected:   {2}\n"
                                             "calculated: {3}".format(name, field, eres, cres))

        dataSource.clear()

        calculated = dataSource.data
        expected = {}
        self.assertEqual(calculated, expected, "cleared data provide\n"
                                               "expected:   {0}\n"
                                               "calculated: {1}".format(expected, calculated))

    def testDataProviderWithOverwrittern(self):
        dataSource = DataProvider()
        dataSource.set('aapl', 'close', 2.0)

        with self.assertRaises(ValueError):
            dataSource.set('aapl', 'close', 3.0)




# -*- coding: utf-8 -*-
u"""
Created on 2015-7-16

@author: cheng.li
"""

import csv
import unittest
import os
import numpy as np
from PyFin.Math.Accumulators.StatefulAccumulators import MovingSharp
from PyFin.Math.Accumulators.StatefulAccumulators import MovingSortino
from PyFin.Math.Accumulators.StatefulAccumulators import MovingDrawdown
from PyFin.Math.Accumulators.StatefulAccumulators import MovingMaxDrawdown


class TestPerformancers(unittest.TestCase):

    def testMovingSharp(self):
        dirName = os.path.dirname(os.path.abspath(__file__))
        filePath = os.path.join(dirName, 'data/sharp.csv')

        with open(filePath, 'r') as fileHandler:
            reader = csv.reader(fileHandler)
            mv = MovingSharp(20, x='ret', y='riskFree')

            for i, row in enumerate(reader):
                if i == 0:
                    continue
                data = {'ret': float(row[1]), 'riskFree': float(row[2])}
                mv.push(data)
                if i == 1:
                    self.assertTrue(np.isnan(mv.result()))
                if i >= 20:
                    calculated = mv.result()
                    expected = float(row[6])
                    self.assertAlmostEqual(calculated, expected, 7, "at index {0:d}\n"
                                                                    "Sharp expected:   {1:f}\n"
                                                                    "Sharp calculated: {2:f}".format(i, expected,
                                                                                                     calculated))

    def testMovingSortino(self):
        dirName = os.path.dirname(os.path.abspath(__file__))
        filePath = os.path.join(dirName, 'data/sortino.csv')

        window = 20

        with open(filePath, 'r') as fileHandler:
            reader = csv.reader(fileHandler)
            mv = MovingSortino(window, x='ret', y='riskFree')
            for i, row in enumerate(reader):
                if i == 0:
                    continue
                mv.push(dict(ret=float(row[2]), riskFree=float(row[3])))
                if i == 1:
                    self.assertTrue(np.isnan(mv.result()))
                if i >= window:
                    calculated = mv.result()
                    expected = float(row[10])
                    self.assertAlmostEqual(calculated, expected, 7, "at index {0:d}\n"
                                                                    "Sortino expected:   {1:f}\n"
                                                                    "Sortino calculated: {2:f}".format(i, expected,
                                                                                                       calculated))

    def testMovingDrawdownIncreasing(self):
        dirName = os.path.dirname(os.path.abspath(__file__))
        filePath = os.path.join(dirName, 'data/drawdown_increasing.csv')

        window = 20

        with open(filePath, 'r') as fileHandler:
            reader = csv.reader(fileHandler)
            mv = MovingDrawdown(window, x='ret')
            for i, row in enumerate(reader):
                if i == 0:
                    continue
                mv.push(dict(ret=float(row[1])))
                expectedDrawdown = float(row[2])
                calculatedDrawdown = mv.result()
                self.assertAlmostEqual(calculatedDrawdown, expectedDrawdown, 10, "at index {0:d}\n"
                                                                                 "Drawdown expected:   {1:f}\n"
                                                                                 "Drawdown calculated: {2:f}".format(i,
                                                                                                                     expectedDrawdown,
                                                                                                                     calculatedDrawdown))

    def testMovingDrawdownDecreasing(self):
        dirName = os.path.dirname(os.path.abspath(__file__))
        filePath = os.path.join(dirName, 'data/drawdown_decreasing.csv')

        window = 20

        with open(filePath, 'r') as fileHandler:
            reader = csv.reader(fileHandler)
            mv = MovingDrawdown(window, x='ret')
            for i, row in enumerate(reader):
                if i == 0:
                    continue
                mv.push(dict(ret=float(row[1])))
                expectedDrawdown = float(row[2])
                calculatedDrawdown = mv.result()
                self.assertAlmostEqual(calculatedDrawdown, expectedDrawdown, 10, "at index {0:d}\n"
                                                                                 "Drawdown expected:   {1:f}\n"
                                                                                 "Drawdown calculated: {2:f}".format(i,
                                                                                                                     expectedDrawdown,
                                                                                                                     calculatedDrawdown))

    def testMovingMaxDrawdownRandom(self):
        dirName = os.path.dirname(os.path.abspath(__file__))
        filePath = os.path.join(dirName, 'data/maxdrawdown_random.csv')

        window = 20

        with open(filePath, 'r') as fileHandler:
            reader = csv.reader(fileHandler)
            mv = MovingMaxDrawdown(window, x='ret')
            for i, row in enumerate(reader):
                if i == 0:
                    continue
                mv.push(dict(ret=float(row[1])))
                expectedDrawdown = float(row[5])
                calculatedDrawdown = mv.result()
                self.assertAlmostEqual(calculatedDrawdown, expectedDrawdown, 7, "at index {0:d}\n"
                                                                                "Max drawdown expected:   {1:f}\n"
                                                                                "Max drawdown calculated: {2:f}".format(
                    i, expectedDrawdown, calculatedDrawdown))



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-8

@author: cheng.li
"""

from PyFin.Env.Settings import Settings
from PyFin.Math.udfs import sign
from PyFin.Utilities.Asserts import require


class IrrFinder(object):

    def __init__(self,
                 leg,
                 npv,
                 dayCounter,
                 comp,
                 freq,
                 includeSettlementDateFlows,
                 settlementDate=None,
                 npvDate=None):
        self._leg = leg
        self._npv = npv
        self._dayCounter = dayCounter
        self._compounding = comp
        self._frequency = freq
        self._includeSettlementDateFlows = includeSettlementDateFlows

        if settlementDate:
            self._settlementDate = settlementDate
        else:
            self._settlementDate = Settings.evaluationDate

        if npvDate:
            self._npvDate = npvDate
        else:
            self._npvDate = self._settlementDate

        self._checkSign()

    def _checkSign(self):
        last_sign = sign(-self._npv)
        sign_changes = 0

        for cf in self._leg:
            if (not cf.hasOccurred(self._settlementDate, self._includeSettlementDateFlows)) \
                    and (not cf.tradingExCoupon(self._settlementDate)):
                this_sign = sign(cf.amount())
                if last_sign * this_sign < 0.:
                    sign_changes += 1
                    break

                if this_sign != 0:
                    last_sign = this_sign

        require(sign_changes == 0,
                ValueError,
                    'given cash flows cannot result in the given market due to sign change')


class CashFlows(object):

    @staticmethod
    def startDate(leg):
        return



